local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local guarddefs = include("sim/unitdefs/guarddefs")


local prism_combatdisguise = 
	{
		name = STRINGS.ABILITIES.DISGUISE,
		
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			local cloakAug = self.cloakAug
			local cost = cloakAug:getTraits().chargeCost
			return abilityutil.formatToolTip ( STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_AUG_USE, util.sformat( STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_AUG_USE_TIP, cost ) )
		end, 

		profile_icon = "gui/icons/Flavour/icon-item_dermalHolo_small.png",
		alwaysShow = true,
		getName = function( self, sim, abilityOwner )
			if abilityOwner and abilityOwner:getTraits().disguiseOn then
				return STRINGS.ABILITIES.DISGUISE_DEACTIVATE
			--	local name = self.abilityOwner:getName()
			--	return name
			else
				return STRINGS.ABILITIES.DISGUISE
			end
		end,

		canUseAbility = function( self, sim, abilityOwner, unit, targetUnit )
			-- Must have a user owner.
			--local userUnit = unit:getUnitOwner()
			if not abilityOwner then
				return false
			end
			
	--		local hasAugment = array.findIf( unit:getChildren(), function( u ) return u:getTraits().charges_clip ~= nil end )  ----- check for AUG
	--		if not hasAugment then 
	--			return false, STRINGS.UI.REASON.NO_POWER_PACK
	--		end

			if not abilityOwner:getTraits().disguiseOn then
			local cloakAug = self.cloakAug
				if cloakAug:getTraits().charges < cloakAug:getTraits().chargeCost then
					return false,  STRINGS.FLAVORED.ITEMS.AUGMENTS.NOT_ENOUGH_CHARGE
				end		
			end
					
			return abilityutil.checkRequirements( unit, abilityOwner )
		end,
		
		onSpawnAbility = function( self, sim, unit )
			self.abilityOwner = unit
			
			local abilityOwner = self.abilityOwner
			for i,childAug in ipairs(abilityOwner:getChildren( )) do
				if childAug:getTraits().isCombatCloak then
					self.cloakAug = childAug
				end
			end
			local cloakAug = self.cloakAug
			cloakAug:getTraits().usesCharges = true
			
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
			if sim:isVersion("0.17.7") then
				sim:addTrigger( simdefs.TRG_UNIT_USEDOOR, self )
			end
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_ALARM_INCREASE, self )
		end,
			
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
			if sim:isVersion("0.17.7") then
				sim:removeTrigger( simdefs.TRG_START_TURN, self )
			end
			sim:removeTrigger( simdefs.TRG_UNIT_USEDOOR, self )
			sim:removeTrigger( simdefs.TRG_ALARM_INCREASE, self )
			self.cloakAug = nil
			self.abilityOwner = nil
		end,

		onTrigger = function ( self, sim, evType, evData )  -- need to get this working

			if evType == simdefs.TRG_UNIT_WARP or evType == simdefs.TRG_UNIT_USEDOOR then
				local abilityOwner = self.abilityOwner

				if abilityOwner and abilityOwner:getTraits().disguiseOn then

					local enemyUnit, range = sim:getQuery().getNearestEnemy( abilityOwner )
				   
					if range <=1.5 then
						abilityOwner:setDisguise(false)	
						abilityOwner:interruptMove( sim ) 		
					end

				end
			elseif evType == simdefs.TRG_START_TURN then
				local abilityOwner = self.abilityOwner
				local player = abilityOwner:getPlayerOwner()
				local cloakAug = self.cloakAug

				if player and sim:getCurrentPlayer() == player and abilityOwner:getTraits().disguiseOn  then
					local x,y abilityOwner:getLocation()
					if cloakAug:getTraits().charges >= cloakAug:getTraits().chargesPerTurn then
						--player:addCPUs( -2, sim, x,y )
						cloakAug:getTraits().charges = (cloakAug:getTraits().charges - cloakAug:getTraits().chargesPerTurn)
						sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=util.sformat( STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_AUG_WARNING,cloakAug:getTraits().chargesPerTurn), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=nil } )
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_MINUS_CHARGE, cloakAug:getTraits().chargesPerTurn), unit = abilityOwner,  color={r=163/255,g=243/255,b=248/255,a=1} })
					else
						abilityOwner:setDisguise(false)
					end
				end
			elseif evType == simdefs.TRG_ALARM_INCREASE then
				local abilityOwner = self.abilityOwner
				local cloakAug = self.cloakAug
				local oldcharges = cloakAug:getTraits().charges
				local lastTrack = cloakAug:getTraits().lastTracker
				local currentTrack = sim:getTracker()
				
				if currentTrack > lastTrack and (cloakAug:getTraits().charges < cloakAug:getTraits().chargesMax) then
					cloakAug:getTraits().charges = math.min(oldcharges + (currentTrack - lastTrack), 10)
					
					local fromCell = sim:getCell( abilityOwner:getLocation() )
					local x0,y0 = fromCell.x, fromCell.y
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_PLUS_CHARGE, (currentTrack - lastTrack)), unit = abilityOwner, color= cdefs.MOVECLR_DEFAULT })
				end
				cloakAug:getTraits().lastTracker = currentTrack
			end
		end,

		
		executeAbility = function( self, sim, abilityOwner )
			--local userUnit = unit:getUnitOwner()

			if abilityOwner:getTraits().disguiseOn then
				abilityOwner:setDisguise(false)
			else

				local kanim = "kanim_guard_male_ftm"
				local wt = util.weighted_list( sim._patrolGuard )	

				for i = 2, #wt, 2 do
					local template = guarddefs[wt[i]]
					if not template.traits.isDrone then
						kanim = template.kanim							
					end

				end
 				
				abilityOwner:setDisguise(true, kanim)
				--abilityOwner:getPlayerOwner():addCPUs( -2 )
				local cloakAug = self.cloakAug
				cloakAug:getTraits().charges = (cloakAug:getTraits().charges - cloakAug:getTraits().chargeCost)
				
				local fromCell = sim:getCell( abilityOwner:getLocation() )
				local x0,y0 = fromCell.x, fromCell.y
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FLAVORED.ITEMS.AUGMENTS.HOLO_MESH_MINUS_CHARGE, cloakAug:getTraits().chargeCost), unit = abilityOwner,  color={r=163/255,g=243/255,b=248/255,a=1} })
			
				if sim:canPlayerSeeUnit( sim:getNPC(), abilityOwner ) then 
					abilityOwner:setDisguise(false)
				end

				abilityOwner:resetAllAiming()
	
				--inventory.useItem( sim, abilityOwner, unit )
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = abilityOwner  } )

		--	sim:processReactions(abilityOwner)			

		end,
	}
return prism_combatdisguise
